/* some C to generate sawteeth of random phase to stdout,
 * printing a log of phases generated to stdout
 * One sawtooth per named file in argc
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define SRATE 44100
#define LENGTH 1
#define NOTE 220
#define PURESAW 1

int main( int argv, char *argc[] )
{
	double buffer[LENGTH*SRATE];
	char bytebuff[2*LENGTH*SRATE];
	int harms = 1+SRATE/(2*NOTE);
	double phase[harms];
	double amp[harms];
	int i, j, k, fd;
	float phi, peak;
	double twopi = asin(1.0)*4.0;
	
	while(--argv) {
		puts( argc[argv] );

		for( i = 1; i < harms; i++ ) {
			/* pseudorandom phase between 0 and 2pi */
			phase[i] = ((float)random())*twopi/((float)RAND_MAX);
			if( PURESAW ) phase[i] = 0.0;
			amp[i] = 1.0/i;
			printf( "  harmonic %i has phase %f and amp %f\n",
				         i, phase[i]/twopi, amp[i] );
		}
	
		phi = peak = 0.0;
		for( i = 0; i < LENGTH*SRATE; i++ ) {
			double sample = 0.0;
		
			for( j = 1; j < harms; j++ ) { /* per harmonic */
				sample += amp[j] * sin( phi*j + phase[j] );
			}
			if( sample > peak ) peak = sample;
			if( -sample > peak ) peak = -sample;
			buffer[i] = sample;

			/* phase increment */
			phi += twopi * NOTE / SRATE;
			if( phi > twopi ) phi -= twopi;
		}	
		printf( "Peak sample value = %f\n", peak );
		for( i = 0; i < LENGTH*SRATE; i++ ) {
			int sample = floor(32767.0*buffer[i]/peak);
			char *sbytes = (char*) &sample;
			bytebuff[i*2] = sbytes[0];
			bytebuff[i*2+1] = sbytes[1];
		}

		fd = creat( argc[argv], S_IRWXU );
		if( fd == -1 ) { puts( "write failed\n" ); continue; }
		printf( "Wrote %d bytes\n\n",
				write( fd, bytebuff, 2*LENGTH*SRATE ) );
		close(fd);
	}
}
